"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockCertificate = void 0;
const precise_proofs_js_1 = require("precise-proofs-js");
const Certificate_1 = require("../blockchain-facade/Certificate");
/*
 *  Extended certificate that is only used in tests
 */
class MockCertificate extends Certificate_1.Certificate {
    async migrateToPublic() {
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const migrationRequestId = await issuerWithSigner.getMigrationRequestId(this.id);
        const migrationRequest = await issuerWithSigner.getMigrationRequest(migrationRequestId);
        const requestor = migrationRequest.owner;
        const privateVolume = this.privateOwnershipCommitment[requestor];
        if (privateVolume.eq(0)) {
            throw new Error(`migrateToPublic(): Requestor doesn't own any private volume in certificate #${this.id}.`);
        }
        const { salts } = await this.getCommitment();
        const calculatedOffChainStorageProperties = this.generateAndAddProofs(this.privateOwnershipCommitment, salts);
        const theProof = precise_proofs_js_1.PreciseProofs.createProof(requestor, calculatedOffChainStorageProperties.leafs, false);
        const onChainProof = theProof.proofPath.map((p) => ({
            left: !!p.left,
            hash: p.left || p.right
        }));
        const { salt } = theProof;
        const tx = await issuerWithSigner.migrateToPublic(migrationRequestId, privateVolume, salt, onChainProof);
        await tx.wait();
        return tx;
    }
    async approvePrivateTransfer(newCommitmentProof) {
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const previousCommitment = this.propertiesDocumentHash;
        const request = await issuerWithSigner.getPrivateTransferRequest(this.id);
        if (!request) {
            throw new Error(`approvePrivateTransfer(): no pending requests to approve.`);
        }
        const theProof = precise_proofs_js_1.PreciseProofs.createProof(request.owner, newCommitmentProof.leafs, false);
        const onChainProof = theProof.proofPath.map((p) => ({
            left: !!p.left,
            hash: p.left || p.right
        }));
        this.propertiesDocumentHash = newCommitmentProof.rootHash;
        const tx = await issuerWithSigner.approvePrivateTransfer(this.id, onChainProof, previousCommitment, newCommitmentProof.rootHash);
        await tx.wait();
        await this.certificateClient.approvePendingOwnershipCommitment(this.id);
        return tx;
    }
}
exports.MockCertificate = MockCertificate;
//# sourceMappingURL=MockCertificate.js.map